using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using TransmissionRssManager.Core;
using TransmissionRssManager.Services;

namespace TransmissionRssManager.Api.Controllers
{
    [ApiController]
    [Route("api/[controller]")]
    public class TorrentsController : ControllerBase
    {
        private readonly ILogger<TorrentsController> _logger;
        private readonly ITransmissionClient _transmissionClient;
        private readonly IConfigService _configService;
        private readonly IPostProcessor _postProcessor;

        public TorrentsController(
            ILogger<TorrentsController> logger,
            ITransmissionClient transmissionClient,
            IConfigService configService,
            IPostProcessor postProcessor)
        {
            _logger = logger;
            _transmissionClient = transmissionClient;
            _configService = configService;
            _postProcessor = postProcessor;
        }

        [HttpGet]
        public async Task<IActionResult> GetTorrents()
        {
            var torrents = await _transmissionClient.GetTorrentsAsync();
            return Ok(torrents);
        }

        [HttpPost]
        public async Task<IActionResult> AddTorrent([FromBody] AddTorrentRequest request)
        {
            var config = _configService.GetConfiguration();
            string downloadDir = request.DownloadDir ?? config.DownloadDirectory;

            var torrentId = await _transmissionClient.AddTorrentAsync(request.Url, downloadDir);
            return Ok(new { id = torrentId });
        }

        [HttpDelete("{id}")]
        public async Task<IActionResult> RemoveTorrent(int id, [FromQuery] bool deleteLocalData = false)
        {
            await _transmissionClient.RemoveTorrentAsync(id, deleteLocalData);
            return Ok();
        }

        [HttpPost("{id}/start")]
        public async Task<IActionResult> StartTorrent(int id)
        {
            await _transmissionClient.StartTorrentAsync(id);
            return Ok();
        }

        [HttpPost("{id}/stop")]
        public async Task<IActionResult> StopTorrent(int id)
        {
            await _transmissionClient.StopTorrentAsync(id);
            return Ok();
        }

        [HttpPost("{id}/process")]
        public async Task<IActionResult> ProcessTorrent(int id)
        {
            var torrents = await _transmissionClient.GetTorrentsAsync();
            var torrent = torrents.Find(t => t.Id == id);
            
            if (torrent == null)
            {
                return NotFound();
            }

            await _postProcessor.ProcessTorrentAsync(torrent);
            return Ok();
        }
    }

    public class AddTorrentRequest
    {
        public string Url { get; set; }
        public string DownloadDir { get; set; }
    }
}